CREATE OR REPLACE package body rob_dashboard
as

REVISION_LABEL        constant varchar2(30) := '$x.y:: 1.0           $';
PACKAGE_NAME          constant varchar2(30) := 'ROB_DASHBOARD';

/**
*  This procedure will create a new HTML Select Option. It should be called within a SELECT element
**/
procedure add_option
( p_value         in varchar2
, p_label         in varchar2
, p_current_value in varchar2
) is
begin
   htp.p
   ( '<OPTION VALUE="'||p_value||'"'
    ||cdwpbase.ifThenElse
      ( nvl(p_current_value,'NULLVALUE') = nvl(p_value,'NULLVALUE')
      , 'SELECTED=Y'
      )
    ||'>'
    ||p_label
    );
end add_option;


/**
*  This procedure will create the HTML Select with the workareas in it.
**/
procedure workarea_select
( p_name          in varchar2 default 'p_workarea_irid'
, p_workarea_irid in number default null -- current value
) is
    cursor c_wa
    is
    select wa.irid
    ,      wa.OWNER
    ,      wa.NAME
    ,      wa.KIND
    ,      wa.DESCRIPTION
    from   sdd_workareas wa
    order
    by     wa.name
    ;

begin
   -- WORKAREA
   htp.tableData
   ( cdwp.add_images('{workarea.gif}')
   ||'<LABEL for='||p_name||'> '||Rob_msg.GetMsg(Rob_msg.CAP077_ODWACHCK_WA,'','','','')||'</LABEL>'
   ,'RIGHT'
   , cattributes=> 'VALIGN="TOP" class="t2"'
   );

   htp.p('<TD ALIGN=LEFT >');
   htp.formselectopen
   ( cname   => p_name
   , cprompt => ''
   , cattributes => 'id='||p_name
   );
   add_option('',Rob_msg.GetMsg(Rob_msg.CAP077_ODWACHCK_WA,'','','',''), p_workarea_irid);
   for r_wa in c_wa loop
     add_option(to_char(r_wa.irid),r_wa.name, p_workarea_irid);
   end loop; -- r_wa
   htp.formselectclose;
   htp.p(odwahelp.help_url( p_him_id => 56));
   htp.p('</TD>');
end workarea_select;


/**
*  This procedure will create the HTML Select with the Repository Users in it.
**/
procedure user_select
( p_name          in varchar2 default 'p_username'
, p_username      in varchar2 default null -- current value
) is
    cursor c_usr
    is
    select usr.username
    ,      usr.full_user_name
    from   sdw_users usr
    where  usr.username <> 'PUBLIC' -- 1.7  PUBLIC is not a real user
    and    usr.user_type = 1  -- 2.4  exclude Roles
    order
    by     full_user_name
    ;
begin
   htp.tableData
   ( cdwp.add_images('{user.gif}')
   ||'Repository User'
   ||'<LABEL for='||p_name||Rob_msg.GetMsg(Rob_msg.CAP280_ODWASRCH_COBY,'','','','')||'</LABEL>'
   ,'RIGHT'
   , cattributes=> 'VALIGN="TOP" class="t2"'
   );

   htp.p('<TD ALIGN=LEFT>');
   htp.formselectopen
   ( cname   => p_name
   , cprompt => ''
   , cattributes => 'id='||p_name
   );
   add_option('',Rob_msg.GetMsg(Rob_msg.CAP281_ODWASRCH_ANYREPUSR,'','','',''), p_username);
   for r_usr in c_usr loop
     add_option(r_usr.username, r_usr.full_user_name, p_username);
   end loop; -- r_usr
   
   htp.formselectclose;
   htp.p(odwahelp.help_url( p_him_id => 86));
   htp.p('</TD>');
exception
  when others
  then
    htp.p(sqlerrm);
end user_select;


/**
*  This procedure will create the HTML Select with the events in it.
**/
procedure mode_select
( p_name          in varchar2 default 'p_mode'
, p_mode          in varchar2 default null -- current value
) is
begin
   htp.tableData
   ( '<LABEL for='||p_name||'> '||'Events'||'</LABEL>'
   ,'RIGHT'
   , cattributes=> 'VALIGN="TOP" class="t2"'
   );
   htp.p('<TD ALIGN=LEFT >');
   htp.formselectopen
   ( cname   => p_name
   , cprompt => ''
   , cattributes => 'id='||p_name
   );
   add_option( '','All', p_mode); 
   add_option( 'CI','Check In', p_mode); 
   add_option( 'CO','Check Out', p_mode); 
   add_option( 'CH','Last Change', p_mode); 
   add_option( 'CR','Creation', p_mode); 
   add_option( 'ME','Merge', p_mode); 
   htp.formselectclose;
   htp.p('</TD>');
end mode_select;

/**
*  This procedure will create the HTML Select with the periods in it.
**/
procedure period_select
( p_name          in varchar2 default 'p_period'
, p_period        in varchar2 default null -- current value
) is
begin
   htp.tableData
   ( '<LABEL for='||p_name||'> '||'Timeframe'||'</LABEL>'
   ,'RIGHT'
   , cattributes=> 'VALIGN="TOP" class="t2"'
   );
   
   htp.p('<TD ALIGN=LEFT >');
   htp.formselectopen
   ( cname   => p_name
   , cprompt => ''
   , cattributes => 'id='||p_name
   );
   add_option( '','All Time', p_period); 
   add_option( 'T','Today', p_period); 
   add_option( 'TY','Today and Yesterday', p_period); 
   add_option( 'Y','Yesterday', p_period); 
   add_option( 'TW','This Week', p_period); 
   add_option( 'TLW','This Week and Last', p_period); 
   add_option( 'LW','Last Week', p_period); 
   add_option( 'TM','This Month', p_period); 
   add_option( 'TLM','This Month and Last', p_period); 
   add_option( 'LM','Last Month', p_period); 
   htp.formselectclose;
   htp.p('</TD>');
end period_select;

/**
*  This procedure will create the HTML Select with the options for the number of results.
**/
procedure count_select
( p_name          in varchar2 default 'p_count'
, p_count         in number default null -- current value
) is
begin
   htp.tableData
   ( '<LABEL for='||p_name||'> '||'Number of Results'||'</LABEL>'
   ,'RIGHT'
   , cattributes=> 'VALIGN="TOP" class="t2"'
   );
   htp.p('<TD ALIGN=LEFT >');
   htp.formselectopen
   ( cname   => p_name
   , cprompt => ''
   , cattributes => 'id='||p_name
   );
   add_option( '10','10 Records', p_count); 
   add_option( '15','15 Records', p_count); 
   add_option( '20','20 Records', p_count); 
   add_option( '30','30 Records', p_count); 
   add_option( '50','50 Records', p_count); 
   add_option( '100','100 Records', p_count); 
   add_option( '200','200 Records', p_count); 
   add_option( '500','500 Records', p_count); 
   add_option( '1000','1000 Records', p_count); 
   htp.formselectclose;
   htp.p('</TD>');
end count_select;

procedure element_type_select
( p_name in varchar2 default 'p_element_type'
, p_element_type in number default null 
) is
    cursor c_ete
    is
     select net.nls_name        name
     ,      et.id               irid
     ,      et.ivid
     from   ci_element_types     cet
     ,      rm_element_types     et
     ,      rm$nls_element_types net
     ,      rm_repositories      rep
     where cet.short_name = et.short_name
     and   net.etid       = et.irid
     and   supertype in ('SHR', 'NSHR')
     and   et.product = 'CI'
     and   et.abstract = 'N'
     and   net.nls_language = rep.nls_language --3.13
     and   exists (
                select null
                from   ck_product_elements pe
                where  pe.element_name = et.name
                and    pe.product_code = 'DESIGN'
                )
     and et.irid not in (5052, 5055)  -- filter out the element types that cannot truly serve as PAC
                                            -- Specific MCO and LOV
     union -- User Extended Types
     select net.nls_name        name
     ,      et.id               irid
     ,      et.ivid
     from   rm_element_types     et
     ,      rm$nls_element_types net
     ,      rm_repositories      rep
     where net.etid       = et.irid
     and   et.product = 'CI'
     and   et.abstract = 'N'
     and   net.nls_language = rep.nls_language --3.13
     and et.user_extension = 'YYY'
    order
    by    1
    ;

begin
          -- Element Type
          htp.tableData
           (cdwp.add_images('{doc.gif}')
          ||'<b><LABEL for='||p_name||'>'||Rob_msg.Getmsg(Rob_msg.CAP217_ODWAROBS_ELTYPE,'','','','')||' </LABEL></b> '
           ,'RIGHT'
           , cattributes=> 'VALIGN="TOP" class="t2"'
           );
          htp.p('<TD ALIGN=LEFT ACOLSPAN=4>');
          htp.formselectopen
          ( cname   => p_name
          , cprompt => ''
          , cattributes => 'id="'||p_name||'"'
          );
          htp.p
          ( '<OPTION VALUE=""'
           ||cdwpbase.ifThenElse
             ( p_Element_Type is null
             , 'SELECTED=Y'
             )
           ||'>'
           ||Rob_msg.GetMsg(Rob_msg.CAP217_ODWAROBS_ELTYPE ,'','','','')
           );
          for r_ete in c_ete loop
            htp.p( '<OPTION VALUE="'||to_char(r_ete.irid)||'"'
               ||cdwpbase.ifThenElse
                 ( r_ete.irid = to_number(p_Element_Type)
                 , 'SELECTED=Y'
                 )
               ||'>'
               ||r_ete.name
               );
            -- Do not display CDM specific objects to general users.
            if (r_ete.irid = 5024) -- and lower(l_cdm_used) = 'y')-- 'BUSINESS FUNCTION'
            then
                -- add Business Rule                    -5024 (+5024 FUN)
                htp.p
                ( '<OPTION VALUE="-5024"'
                ||cdwpbase.ifThenElse
                  ( -5024 = to_number(p_Element_Type)
                  , 'SELECTED=Y'
                  )
                ||'>'
                ||'Business Rule (Function)'
                );
                -- add Business Rule Design Definition  -4907 (+4907 PLM)
                htp.p
                ( '<OPTION VALUE="-4907"'
                ||cdwpbase.ifThenElse
                  ( -4907 = to_number(p_Element_Type)
                  , 'SELECTED=Y'
                  )
                ||'>'
                ||'Business Rule Design Definition'
                );
            end if; -- r_ete.irid = 5024 -- 'BUSINESS FUNCTION'
          end loop; -- r_ete
          htp.formselectclose;
          htp.p(odwahelp.help_url( p_him_id => 52));
          htp.p('</TD>');
 end;

/*
SELECT DISTINCT
         e.irid
  ,      e.ivid
  ,      e.name
  ,      decode
         ( e.state
         , 'N', '-'
         , 'O', odwavrsn.get_version_label( e.ivid)
         , e.vlabel
         ) 
  ,      nvl( e.date_changed, e.date_created) date_changed 
  ,      e.logical_type_id 
  ,      e.wastebasket 
  ,      vatip.notes
  FROM   I$SDD_OBJECT_VERSIONS        e 
  ,      I$SDD_BRANCHES               be
  ,      I$SDD_VERSION_ASSOCIATIONS   vatip
  WHERE  e.branch_id = be.branch_id(+) 
  AND    exists ( select 'x'
                         from   i$sdd_folder_members fmo
                         ,      sdw_access_rights    acc
                         where  fmo.member_object     = e.irid
                         and    fmo.folder_reference  = acc.object_reference
                         and    acc.grantee_reference in ( USER,'PUBLIC')
                         UNION
                         select 'x'
                         from   sdw_access_rights    acc
                         where  e.irid                = acc.object_reference
                         and    acc.grantee_reference in ( USER,'PUBLIC')
                       )
  AND    e.wastebasket = 'N'
  AND    vatip.from_ivid(+) = e.ivid 
  AND    nvl( e.changed_by, e.created_by)  = p_username
  AND    
  (nvl( e.date_changed, e.date_created) >=  to_date('10-MAY-2005' , 'DD-MON-YYYY')
     OR
     va.date_created >=  to_date('12-MAY-2004' , 'DD-MON-YYYY')
  )
 
  ORDER
  BY     e.name
  ,      e.irid 
*/

procedure show_recent_events
( p_workarea_irid in number   
, p_username      in varchar2 
, p_count         in number   
, p_period        in varchar2 
, p_mode          in varchar2 
, p_order_by      in varchar2 
, p_irid          in number   default null -- only events for a specific object
, p_element_type  in number   default null -- only events for a specific object type
) is
  l_from_date date:= sysdate-5000;
  l_to_date date:= sysdate+5000;

  procedure write_row
  ( p_ivid in number
  , p_name in varchar2
  , p_date_event in date
  , p_user_event in varchar2
  , p_logical_type_id in number
  , p_irid in number
  , p_check_state in varchar2
  , p_vlabel in varchar2
  , p_event_type in varchar2
  , p_notes in varchar2
  , p_workarea in varchar2
  )  is
  begin
        cdwp.tableRowOpen;
        cdwp.tableDataValue(p_event_type); -- event
        cdwp.tableDataValue(' '); -- wastebasket gif
        cdwp.tableDataValue
		(cdwpbase.ifThenElse
          ( p_check_state in ('O', 'L')
            , cdwp.add_images
              ( '{check_mark.gif}'
              )
          )
	    ); -- Check Out Mark
        cdwp.tableDataValue('<A href="javascript:go('''||p_ivid||''','''||p_name||''',false);">'||p_name||'</A>'); -- object name
        -- launchpad with icons for VHV, INFO, Dashboard and Object Browser
        cdwp.tableDataValue
        ( odwavrsn.url_for_vhv(p_ivid => p_ivid, p_js_included => true, p_ivid_only => true)
        ||odwainfo.info_url( p_ivid => p_ivid)
        ||rob_dashboard.get_dashboard_link(p_irid => p_irid)
        ||htf.anchor2
          ( curl => 'javascript:browse('''||to_char(p_ivid)||''');'
             , ctext =>  cdwp.add_images
                          ( p_text => '{browsemed.gif}'  -- browser + gray
                          , p_attributes=>' Awidth="30"'
                          ||'alt="Go to Object Browser"'
                        )
             )
        ); 
        cdwp.tableDataValue(p_vlabel); -- version label
        cdwp.tableDataValue(jr_name.get_path( id => jr_acc_rights.get_owning_container(obj_irid => p_irid), format=> 'NAME')); -- path
        cdwp.tableDataValue(p_workarea); -- workarea
        cdwp.tableDataValue(cdwp.add_images( '{'||cdwpbase.get_icon(p_logical_type_id)||'}')
		                 ||cdwpbase.get_nls_type_name( p_type_id => p_logical_type_id)); -- type
  --      cdwp.tableDataValue(' '); -- type
        cdwp.tableDataValue(to_char(p_date_event,'DD-MON-YYYY HH24:MI')); -- date event
        cdwp.tableDataValue(p_user_event); -- user event
        cdwp.tableDataValue(''); 
        cdwp.tableDataValue(p_notes); -- notes
        cdwp.tableRowClose;
  end write_row;
  
begin
  -- we want to show the following properties:
  --   object name and type (hyperlink to report)
  --   folder path
  --   version label, current state, event notes (CI or CO notes)
  --   date of event, user who performed the event
  --   list of workareas?
  --   links to Browser and VHV and INFo
  -- most of this can be seen in Search
  if p_period is not null
  then
      case p_period
      when 'T' -- Today
      then l_from_date:= trunc(sysdate);
           l_to_date:= trunc(sysdate);
      when 'TY' -- Today and Yesterday
      then l_from_date:= trunc(sysdate-1);
           l_to_date:= trunc(sysdate);
      when 'Y'  -- yesterday
      then l_from_date:= trunc(sysdate-1);
           l_to_date:= trunc(sysdate-1);
      when 'TW' -- this week : go back to last Sunday; note: to_char(sysdate,'D' ) = 1 for SUNDAY; go back as many days as the number of the current week day minus one 
      then l_from_date:= trunc(sysdate - to_number(to_char(sysdate,'D')) +1 );
           l_to_date:= trunc(sysdate);
      when 'TLW' -- this week and last: go back to two Sundays ago ; note: to_char(sysdate,'D' ) = 1 for SUNDAY; go back as many days as the number of the current week day minus one 
      then l_from_date:= trunc(sysdate- 7 - to_number(to_char(sysdate,'D'))+1 );
           l_to_date:= trunc(sysdate);
      when 'LW' -- last week : go back to two Sundays ago 
      then l_from_date:= trunc(sysdate- 7 - to_number(to_char(sysdate,'D'))+1 );
           l_to_date:= trunc(sysdate - to_number(to_char(sysdate,'D'))+1 );
      when 'TM' -- this month so go the first day of the current month
      then l_from_date:= trunc(sysdate- to_number(to_char(sysdate,'DD'))+1 ); 
           l_to_date:= trunc(sysdate);
      when 'TLM' -- this month so go the first day of the previous month
      then l_from_date:= sysdate - to_number(to_char(sysdate,'DD')) 
                   - to_number(to_char (sysdate- to_number(to_char(sysdate,'DD')) ,'DD'))+1; 
           l_to_date:= trunc(sysdate);
      when 'LM' -- Last Monthy 
      then l_from_date:= sysdate - to_number(to_char(sysdate,'DD')) 
                   - to_number(to_char (sysdate- to_number(to_char(sysdate,'DD')) ,'DD'))+1; 
           l_from_date:= trunc(sysdate- to_number(to_char(sysdate,'DD'))); 
      end case;
  end if;  
      cdwp.tableOpen;

        cdwp.tableRowOpen;
        cdwp.tableDataValue
        ( '<font face="Arial Narrow"><font size=-1><b>'
        ||'<A HREF="javascript:document.queryForm.p_order_by.value = ''event'
           ||cdwpbase.ifThenElse
             ( instr(lower(p_Order_By), 'asc') > 0
             , 'desc'
             , 'asc'
             )
           ||''';document.queryForm.submit();">'||'Event'||'</A>'
        ||'</b></font></font>'
        );
        cdwp.tableDataValue(' '); -- wastebasket gif
        cdwp.tableDataValue(' '); -- check mark gif
        cdwp.tableDataValue
        ( '<font face="Arial Narrow"><font size=-1><b>'
        ||'<A HREF="javascript:document.queryForm.p_order_by.value = ''name '
           ||cdwpbase.ifThenElse
             ( instr(lower(p_Order_By), 'asc') > 0
             , 'desc'
             , 'asc'
             )
           ||''';document.queryForm.submit();">'||Rob_msg.Getmsg(Rob_msg.CAP317_ODWASRCH_ELEMENT,'','','','')||'</A>'
        ||'</b></font></font>'
        );
        cdwp.tableDataValue(' '); -- actions/info
        cdwp.tableDataValue('<font face="Arial Narrow"><font size=-1><b>'||Rob_msg.Getmsg(Rob_msg.CAP203_ODWAPROP_VER,'','','','')||'</b></font></font>');
--        cdwp.tableDataValue(cdwpbase.nbsp(2));
  --      cdwp.tableDataValue('<font face="Arial Narrow"><font size=-1><b>Path</b></font></font>');
        cdwp.tableDataValue
        ( '<font face="Arial Narrow"><font size=-1><b>'
        ||'<A HREF="javascript:document.queryForm.p_order_by.value = ''path '
           ||cdwpbase.ifThenElse
             ( instr(lower(p_order_by), 'asc') > 0
             , 'desc'
             , 'asc'
             )
           ||''';document.queryForm.submit();">'||Rob_msg.Getmsg(Rob_msg.CAP061_ODWA_PATH,'','','','')||'</A>'
        ||'</b></font></font>'
        );
        cdwp.tableDataValue
        ( '<font face="Arial Narrow"><font size=-1><b>'
        ||'<A HREF="javascript:document.queryForm.p_order_by.value = ''workarea '
           ||cdwpbase.ifThenElse
             ( instr(lower(p_order_by), 'asc') > 0
             , 'desc'
             , 'asc'
             )
           ||''';document.queryForm.submit();">'||'Workarea'||'</A>'
        ||'</b></font></font>'
        );
  --  P.Ebell -- Check op gElementType tijdelijk verwijderd, Lucas raadplegen
  --      if qElementType is null
  --      then
          cdwp.tableDataValue
          ('<font face="Arial Narrow"><font size=-1><b>'
        ||'<A HREF="javascript:document.queryForm.p_order_by.value = ''type '
           ||cdwpbase.ifThenElse
             ( instr(lower(p_Order_By), 'asc') > 0
             , 'desc'
             , 'asc'
             )
           ||''';document.queryForm.submit();">'||Rob_msg.Getmsg(Rob_msg.CAP045_CDWP_TYPE,'','','','')||'</A>'
          ||' </b></font></font>'
          );
  --      end if;
  --      cdwp.tableDataValue('<font face="Arial Narrow"><font size=-1><b>Date Modified</b></font></font>');
        cdwp.tableDataValue
        ( '<font face="Arial Narrow"><font size=-1><b>'
        ||'<A HREF="javascript:document.queryForm.p_order_by.value = ''touched '
           ||cdwpbase.ifThenElse
             ( instr(lower(p_Order_By), 'touched') > 0  -- if already ordered by touched, just swap the order
             , cdwpbase.ifThenElse
               ( instr(lower(p_Order_By), 'asc') > 0
               , 'desc'
               , 'asc'
               )
             , 'desc'   -- the default/first ordering on touched should be DESCENDING (most recently modified on top)
             )
           ||''';document.queryForm.submit();" title="'||to_char(l_from_date,'DD-MM-YYYY')||' to '||to_char(l_to_date,'DD-MM-YYYY')||'" >'||'Date/Time Event'||'</A>'
        ||'</b></font></font>'
        );
        cdwp.tableDataValue
        ( '<font face="Arial Narrow"><font size=-1><b>'
        ||'<A HREF="javascript:document.queryForm.p_order_by.value = ''user '
           ||cdwpbase.ifThenElse
             ( instr(lower(p_Order_By), 'user') > 0  -- if already ordered by touched, just swap the order
             , cdwpbase.ifThenElse
               ( instr(lower(p_Order_By), 'asc') > 0
               , 'desc'
               , 'asc'
               )
             , 'desc'   -- the default/first ordering on touched should be DESCENDING (most recently modified on top)
             )
           ||''';document.queryForm.submit();">'||'Acting User '||'</A>'
        ||'</b></font></font>'
        );


        cdwp.tableDataValue
        ('<font face="Arial Narrow"><font size=-1><b>'
        ||''
        ||' </b></font></font>'
        );
        cdwp.tableDataValue
        ('<font face="Arial Narrow"><font size=-1><b>'
        ||Rob_msg.GetMsg(Rob_msg.CAP0152_CDWP_NOTES,'','','','')
        );
        cdwp.tableRowClose;

		/* EDGE_KIND in I$SDD_VERSION_ASSOCIATIONS. 
		Real transition types are: N(ormal of Node), B(ranch), M(erge), C(heck Out) L(ocked Check Out). 
		Special types of transitions have been defined to indicate the Root version of a tree (R) and the Tip Version of a branch (T). 
		Note: each branch has a tip; the tip is the latest checked in version on that branch.
				
		Looking for events:
		
		Creation  
		Check In
		Check Out
		Change
		Merge
		Branch
		*/
  for obj in 
  (
  select *
  from (SELECT DISTINCT
         irid
  ,      ivid
  ,      name
  ,      check_state
  ,      vlabel
  ,      date_event 
  ,      user_event
  ,      Event_Type 
  ,      logical_type_id 
  ,      cdwpbase.get_nls_type_name( logical_type_id) logical_type_name
  ,      jr_name.get_path( jr_acc_rights.get_owning_container( irid),  'NAME') path
  ,      wastebasket 
  ,      notes
  ,      edge_kind
  ,      case check_state
         when 'N'
		 then (select wa.name from i$sdd_workareas wa, i$sdd_wa_context wac where wac.OBJECT_IVID = ivid and wa.irid = wac.WORKAREA_IRID)
		 when 'O'
		 then (select wa.name from i$sdd_workareas wa, i$sdd_wa_context wac where wac.OBJECT_IVID = ivid and wa.irid = wac.WORKAREA_IRID)
		 else ''
		 end   Workarea		  
  FROM   ( -- first query: Check Out, Check In events 
           select e.irid
           ,      e.ivid
           ,      e.name
           ,      e.state check_state
           ,      decode
                  ( e.state
                  , 'N', '-'
                  , 'O', odwavrsn.get_version_label( e.ivid)
                  , e.vlabel
                  ) vlabel
           ,      nvl(va.date_changed, va.date_created) date_event 
           ,      nvl(va.changed_by, va.created_by) user_event
           ,      case
                  when  va.edge_kind in ('C','L')
                  then  'Check Out'
                  when  va.edge_kind in ('N','T','R') and e.state='I' 
                  then  'Check In'
                  when  va.edge_kind in ('M') 
                  then  'Merge'
                  end   Event_Type 
           ,      e.logical_type_id 
           ,      e.wastebasket 
           ,      va.notes
                  ||case va.edge_kind
                    when 'M' 
                    then ' ;Merge from version '||odwavrsn.get_version_label( va.from_ivid) 
                    end notes
           ,      va.edge_kind 
           from   I$SDD_OBJECT_VERSIONS        e 
           ,      I$SDD_BRANCHES               be
           ,      I$SDD_VERSION_ASSOCIATIONS   va
           WHERE  e.branch_id = be.branch_id(+) 
           AND    va.to_ivid = e.ivid 
           and    nvl(va.edge_kind,'N')  in ('N','C','L','M','R')
           and    ( p_mode is null
                    or
                    p_mode =  case
                              when  va.edge_kind in ('C','L')
                              then  'CO'
                              when  va.edge_kind in ('N','T','R') and e.state='I' 
                              then  'CI'
                              when  va.edge_kind in ('M')
                              then  'ME'
                              end   
                 ) 
           UNION ALL -- check outs  for currently checked in versions
           select src.irid
           ,      src.ivid
           ,      src.name
           ,      src.state check_state
           ,      odwavrsn.get_version_label( src.ivid)||'CO'  vlabel
           ,      va.date_created  date_event 
           ,      va.created_by user_event
           ,      'Check Out'   Event_Type 
           ,      src.logical_type_id 
           ,      src.wastebasket 
           ,      'n/a'  Notes
           ,      va.edge_kind 
           from   I$SDD_OBJECT_VERSIONS        src
           ,      I$SDD_BRANCHES               be
           ,      I$SDD_VERSION_ASSOCIATIONS   va
           WHERE  src.branch_id = be.branch_id(+) 
           and    va.from_ivid = src.ivid
           AND    va.created_by = nvl (p_username, va.created_by)
           and    (p_mode is null or p_mode='CO')
           and va.edge_kind='N'
           UNION ALL -- most recent change for currently checked in, new or checked out objects
           select e.irid
           ,      e.ivid
           ,      e.name
           ,      e.state check_state
           ,      odwavrsn.get_version_label( e.ivid)  vlabel
           ,      to_date(substr(e.comments, 1, 19),'dd-mm-yyyy hh24:mi:ss')  date_event 
           ,      substr(e.comments, 20) user_event
           ,      'Change'   Event_Type 
           ,      e.logical_type_id 
           ,      e.wastebasket 
           ,      ''  Notes
           ,      va.edge_kind 
           from   I$SDD_OBJECT_VERSIONS        e
           ,      I$SDD_BRANCHES               be
           ,      I$SDD_VERSION_ASSOCIATIONS   va
           WHERE  e.branch_id = be.branch_id(+) 
           and    va.to_ivid = e.ivid
           and    (p_mode is null or p_mode='CH')
           and    va.edge_kind in ('C', 'L', 'N','R')
           UNION ALL -- creation 
           select e.irid
           ,      e.ivid
           ,      e.name
           ,      e.state check_state
           ,      odwavrsn.get_version_label( e.ivid)  vlabel
           ,      e.date_created  date_event 
           ,      e.created_by user_event
           ,      'Creation'   Event_Type 
           ,      e.logical_type_id 
           ,      e.wastebasket 
           ,      ''  Notes
           ,      ''
           from   I$SDD_OBJECT_VERSIONS        e
           ,      I$SDD_BRANCHES               be
           ,      I$SDD_VERSION_ASSOCIATIONS   va
           WHERE  e.branch_id = be.branch_id(+) 
           and    va.to_ivid(+) = e.ivid
           AND    e.created_by = nvl (p_username, e.created_by)
           and    (va.edge_kind is null and e.state='N'
                   OR
                   va.edge_kind ='R'
                  )
           and    (p_mode is null or p_mode='CR')
           ) obj
  where  exists ( select 'x'
                  from   i$sdd_folder_members fmo
                  ,      sdw_access_rights    acc
                  where  fmo.member_object     = obj.irid
                  and    fmo.folder_reference  = acc.object_reference
                  and    acc.grantee_reference in ( USER,'PUBLIC')
                  UNION
                  select 'x'
                  from   sdw_access_rights    acc
                  where  obj.irid                = acc.object_reference
                  and    acc.grantee_reference in ( USER,'PUBLIC')
                )
--  AND    e.wastebasket = 'N'
  AND    obj.user_event = nvl (p_username, obj.user_event)
  and    obj.irid = nvl(p_irid, obj.irid)
  AND    ( p_workarea_irid is null
           OR
	   exists (select 1  
	           from   I$SDD_WA_CONTEXT wac
                   where  wac.workarea_irid = p_workarea_irid 
                   AND    wac.object_ivid = obj.ivid
                  )
         ) 
  AND    trunc(date_event) between l_from_date and l_to_date
  AND    logical_type_id = nvl(p_element_type, logical_type_id) 
  ORDER
  BY     case p_order_by
         when null
         then sysdate + (sysdate - date_event )
         when 'touched desc'
         then sysdate + (sysdate - date_event )
         when 'touched asc'
         then date_event 
         end
  ,      case p_order_by
         when 'path desc'
         then path
         when 'name desc'
         then obj.name
         when 'eventdesc'
         then event_type
         when 'type desc'
         then logical_type_name 
         when 'user desc'
         then user_event
         when 'workarea desc'
         then workarea
         end desc 
  ,      case p_order_by
         when 'path asc'
         then path
         when 'name asc'
         then obj.name
         when 'eventasc'
         then event_type
         when 'type asc'
         then logical_type_name 
         when 'user asc'
         then user_event
         when 'workarea asc'
         then workarea
         end asc
  ,      date_event desc
  ,      name
  ,      vlabel
  ) 
  where rownum < p_count 
) loop
  /*
    if check_state = N or O or L then we can find a single Workarea linked to the object version
    if check state = I, O or L, we can find the last object change in obj.last_date_changed and obj.last_changed_by
    if check state = I, we can find the check out date from VA_date_created and va_created_by
  */
  write_row
  ( p_ivid => obj.ivid
  , p_name => obj.name
  , p_date_event => obj.date_event
  , p_user_event => obj.user_event
  , p_logical_type_id => obj.logical_type_id
  , p_irid => obj.irid
  , p_check_state => obj.check_state
  , p_vlabel => obj.vlabel
  , p_event_type  => obj.event_type
  , p_notes  => obj.notes
  , p_workarea => obj.workarea
  );
end loop;
    cdwp.tableClose;
end show_recent_events;

procedure recent_event
( p_workarea_irid in number   default null
, p_username      in varchar2 default null
, p_count         in number   default 15
, p_period        in varchar2 default null -- allowable values:  T, TY, Y, TW, TLW, LW, TM, TLM, LM for Today, Yesterday and Today, Yesterday, This Week, This Week and Last, This Month, This Month and Last
, p_mode          in varchar2 default null -- allowable values: CI, CO, CH for Check In, Check Out, Change; null means: all
, p_order_by      in varchar2 default null -- allowable values: USER, DATE, NAME, FOLDER, TYPE
, p_irid          in number   default null -- only events for a specific object
, p_element_type  in number   default null -- only events for a specific object type
) is
begin
  -- create session when necessary, refresh session cache, set workarea context
  -- from here on reference to odwactxt.get_... functions for any context value
  odwactxt.update_context
  ( p_package_name  => PACKAGE_NAME
  , p_procedure_name=> 'recent_elements'
  , p_folder_ivid   => null
  , p_cfg_ivid      => null
  , p_workarea_irid => p_workarea_irid
  );

  htp.htmlOpen;
  htp.headOpen;
  cdwp.write_about(package_name, revision_label);
  htp.title('ROB Dashboard - Overview of most recent events');
     odwainfo.js_invoke_info;
     -- JavaScript function to submit the download file request form
     cdwp_fil.js_download_file_request;
     odwaupld.js_upload_request;
     odwaupld.js_crefol_request;
     cdwp.include_report_styles;
     -- JavaScript function to do the Check Out request
     odwachck.js_checkout_request;
     odwachki.js_checkin_request;
     odwachck.js_undocheckout_request;
     odwabrow.js_invoke_browser;
     odwavrsn.js_invoke_vhv_ivid_only;
    odwa_fol.js_go(p_session_id => odwactxt.get_session_id,p_target => '_NEW');
  htp.headClose;
  htp.bodyOpen;
  cdwp.tool_header(p_tool => 'DASHBOARD');

  htp.formopen
  ( curl         => 'rob_dashboard.recent_event'
  , cmethod      => 'GET'
  , cattributes  => 'NAME="queryForm" '
  , ctarget      => ''
  );
  htp.FORMHIDDEN(cname=>'p_order_by' , cvalue=>p_order_by);
  htp.FORMHIDDEN(cname=>'p_irid' , cvalue=>p_irid);

  htp.tableOpen;
  htp.tableRowOpen;
  htp.p('<TD>');
  -- open table with the Search Input fields
  cdwp.tableOpen
  ( p_parameters => 'ALIGN="LEFT" BORDER="0" '
  );

  cdwp.tableRowOpen;
  workarea_select( p_workarea_irid => p_workarea_irid);
  period_select( p_period => p_period);
  element_type_select( p_element_type => p_element_type); 
  cdwp.tableRowClose;

  cdwp.tableRowOpen;
  user_select( p_username => p_username);
  mode_select( p_mode => p_mode);
  count_select( p_count => p_count);
  cdwp.tableRowClose;

  cdwp.tableRowOpen;
  htp.p
  (' <TD BGCOLOR="#FFFFFF" <A href="javascript:document.queryForm.submit()'
  ||' ">'
  ||cdwp.add_images( '{search_round.gif}')
  ||'</A></TD>'
  );
  cdwp.tableRowClose;

  cdwp.tableClose;
  htp.p('</TD>');
  htp.tableRowClose;
  htp.tableRowOpen;
  htp.p('<TD>');

  htp.formClose();
  htp.p('<BR />');
  -- from here we can start writing query results
  show_recent_events 
  ( p_workarea_irid => p_workarea_irid
  , p_username      => p_username 
  , p_count         => p_count 
  , p_period        => p_period 
  , p_mode          => p_mode
  , p_order_by      => p_order_by 
  , p_irid          => p_irid
  , p_element_type  => p_element_type
  );
  htp.p('</TD>');
  htp.tableRowClose;
  htp.tableClose;
  htp.bodyClose;
  htp.htmlClose;
exception
  when others
  then
    htp.p(sqlerrm);
  htp.bodyClose;
  htp.htmlClose;
end recent_event;


function get_dashboard_link
( p_workarea_irid in number   default null
, p_username      in varchar2 default null
, p_count         in number   default 15
, p_period        in varchar2 default null -- allowable values:  T, TY, Y, TW, TLW, LW, TM, TLM, LM for Today, Yesterday and Today, Yesterday, This Week, This Week and Last, This Month, This Month and Last
, p_mode          in varchar2 default null -- allowable values: CI, CO, CH for Check In, Check Out, Change; null means: all
, p_order_by      in varchar2 default null -- allowable values: USER, DATE, NAME, FOLDER, TYPE
, p_irid          in number   default null -- only events for a specific object
, p_element_type  in number   default null -- only events for a specific object type
) return varchar2
is
  l_url varchar2(4000):='';
begin
  l_url:= cdwpbase.ifThenElse
          ( p_workarea_irid is not null
          , chr(38)||'p_workarea_irid='||p_workarea_irid
          )
       || cdwpbase.ifThenElse
          ( p_username is not null
          , chr(38)||'p_username='||p_username
          )
       || cdwpbase.ifThenElse
          ( p_count is not null
          , chr(38)||'p_count='||p_count
          )
       || cdwpbase.ifThenElse
          ( p_period is not null
          , chr(38)||'p_period='||p_period
          )
       || cdwpbase.ifThenElse
          ( p_mode is not null
          , chr(38)||'p_mode='||p_mode
          )
       || cdwpbase.ifThenElse
          ( p_order_by is not null
          , chr(38)||'p_order_by='||p_order_by
          )
       || cdwpbase.ifThenElse
          ( p_irid is not null
          , chr(38)||'p_irid='||p_irid
          )
          ;
  return '<a href="rob_dashboard.recent_event'
         ||cdwpbase.ifThenElse
           ( length(l_url)>0
           , '?'||substr(l_url,2)  -- remove the first ampersand
           )
         ||'">'||cdwp.add_images('{dashboard_icon.gif}')||'</a>';
end get_dashboard_link;


end;
/
